#!/bin/sh

ARCH=""

detect_version() {
    if [ -e /etc/ant_version ]; then
        if [ ! -f version ]; then
            echo "RUNME: ERR FW!!!" >>/tmp/upgrade_result
            rm * -rf >/dev/null 2>&1
            cd ..
            rmdir /tmp/mem/fw
            umount -l /tmp/mem >/dev/null 2>&1
            rmdir /tmp/mem/
            echo 6 > /tmp/ret
            exit 6
        fi
        nVer=$(cat version)
        oVer=$(cat /etc/ant_version)

        if [ ${nVer} -lt ${oVer} ]; then
            echo "RUNME: Cannot Downgrade!!!" >>/tmp/upgrade_result
            rm * -rf >/dev/null 2>&1
            cd ..
            rmdir /tmp/mem/fw
            umount -l /tmp/mem >/dev/null 2>&1
            rmdir /tmp/mem/
            echo 7 > /tmp/ret
            exit 7
        fi
    fi
}

get_arch() {
    if [ "$(cat </proc/cpuinfo | grep rv64)" != "" ]; then
        ARCH="riscv"
    elif [ "$(cat </proc/cpuinfo | grep ARMv7)" != "" ]; then
        ARCH="arm"
    else
        ARCH="unknown"
    fi
}

flash_arm() {
    if [ -e BOOT.bin ]; then
        flash_erase /dev/mtd0 0x0 0x40 >/dev/null 2>&1
        nandwrite -p -s 0x0 /dev/mtd0 BOOT.bin >/dev/null 2>&1
    fi

    if [ -e devicetree.dtb ]; then
        flash_erase /dev/mtd0 0x1A00000 0x1 >/dev/null 2>&1
        nandwrite -p -s 0x1A00000 /dev/mtd0 devicetree.dtb >/dev/null 2>&1
    fi

    if [ -e uImage ]; then
        flash_erase /dev/mtd0 0x2000000 0x40 >/dev/null 2>&1
        nandwrite -p -s 0x2000000 /dev/mtd0 uImage >/dev/null 2>&1
    fi

    if [ -e uramdisk.image.gz ]; then
        md5=$(md5sum uramdisk.image.gz | awk '{print $1}')
        md5_r=$(cat md5_info)
        if [ $md5 == $md5_r ]; then
            flash_erase /dev/mtd1 0x0 0x100 >/dev/null 2>&1
            nandwrite -p -s 0x0 /dev/mtd1 uramdisk.image.gz >/dev/null 2>&1
            if [ -e /dev/mtd4 ]; then
                flash_erase /dev/mtd4 0x0 0x100 >/dev/null 2>&1
                nandwrite -p -s 0x0 /dev/mtd4 uramdisk.image.gz >/dev/null 2>&1
            fi
        else
            echo $md5 >/config/md5_error
            echo $md5_r >>/config/md5_error
            echo "Error md5! $md5 $md5_r" >>/tmp/upgrade_result
            echo 1 > /tmp/ret
            exit 1
        fi
    fi

    exit 0
}

flash_riscv() {
    rm -rf uImage uramdisk.image.gz BOOT.bin
    tar -zxf fw-1880-*.tar.gz

    if [ -e bl1.bin ]; then
        echo " "
        echo "--- erase bl1 ---"
        flash_erase /dev/mtd0 0x40000 2

        echo " "
        echo "--- write bl1 ---"
        nandwrite -p -s 0x40000 /dev/mtd0 bl1.bin

        echo " "
        echo "--- write bl1 backup ---"
        nandwrite -p -s 0x60000 /dev/mtd0 bl1.bin
    fi

    if [ -e bl2.bin ]; then
        echo " "
        echo "--- erase bl2 ---"
        flash_erase /dev/mtd0 0x80000 0 >bbt2.log

        echo " "
        echo "--- write bl2 ---"
        nandwrite -p -s 0x80000 /dev/mtd0 bl2.bin
        upgrade_vector vector.bin $(ls -l bl2.bin | awk '{print $5}') 4 \
            $(cat bbt2.log | grep Skipping | sed -n 's/.*\Skipping bad block at\(.*\)\.*/\1/p' | xargs)
        flash_erase /dev/mtd0 0 2
        nandwrite -p -s 0 /dev/mtd0 vector.bin
        nandwrite -p -s 0x20000 /dev/mtd0 vector.bin
    fi

    exit 0
}

detect_version
get_arch

if [ "$ARCH" == "riscv" ]; then
    flash_riscv
elif [ "$ARCH" == "arm" ]; then
    flash_arm
else
    echo "error! unknown architecture"
    echo 2 > /tmp/ret
    exit 2
fi

rm -rf /config/scanfreqdone 2>/dev/null
sync >/dev/null 2>&1

exit 0
